/* Playlist.h */

/* Copyright (C) 2011  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


/*
 * Playlist.h
 *
 *  Created on: Apr 6, 2011
 *      Author: luke
 */

#ifndef PLAYLISTHANDLER_H_
#define PLAYLISTHANDLER_H_

#include "Helper/globals.h"
#include "Helper/MetaData/MetaData.h"
#include "Helper/PlaylistMode.h"
#include "Helper/SayonaraClass.h"
#include "Helper/Shutdown/Shutdown.h"

#include "Components/Engine/EngineHandler.h"
#include "Components/PlayManager/PlayManager.h"
#include "Database/DatabaseConnector.h"

#include "PlaylistLoader.h"
#include "Playlist.h"

#include <QTimer>
#include <memory>


typedef std::shared_ptr<Playlist> PlaylistPtr;
typedef std::shared_ptr<const Playlist> PlaylistConstPtr;

class PlaylistHandler : public QObject, protected SayonaraClass {

	Q_OBJECT

	SINGLETON_QOBJECT(PlaylistHandler)

public:

	enum class PlaylistIndex : quint8
	{
		Current=0,
		Active
	};


signals:
	void sig_playlist_created(const PlaylistPtr& pl);
	void sig_cur_track_idx_changed(int track_idx, int playlist_idx);

	void sig_new_playlist_added(int idx);
	void sig_playlist_name_changed(int idx);
	void sig_playlists_changed();


public:

	MetaDataList get_tracks(PlaylistIndex which);

	// playlist operations
	void clear_playlist();
	void play_next(const MetaDataList&);
	void insert_tracks(const MetaDataList&, int idx, int pl_idx);
	void append_tracks(const MetaDataList&, int pl_idx);
	void move_rows(const IdxList&, int tgt_idx, int pl_idx);
	void remove_rows(const IdxList&, int pl_idx);


	// gui selection changed
	void selection_changed(const IdxList&);

	// change track triggered by user
	void change_track(int idx, int playlist_idx);

	// playlist idx operations
	int	change_playlist_index(int idx);
	int	get_active_idx() const;
	int	get_current_idx() const;

	PlaylistConstPtr get_playlist_at(int idx) const;

	// create playlists
	int create_playlist(const QStringList&, QString name=QString(), bool temporary=true, Playlist::Type type=Playlist::Type::Std);
	int create_playlist(const MetaDataList&, QString name=QString(), bool temporary=true, Playlist::Type type=Playlist::Type::Std);
	int create_playlist(const QString& dir, QString name=QString(), bool temporary=true, Playlist::Type type=Playlist::Type::Std);
	int create_playlist(const CustomPlaylist& pl);
	int create_empty_playlist(const QString& name);

	// called by create playlists

	void delete_playlist(int idx);
	void close_playlist(int idx);

	// requests new name
	QString request_new_playlist_name() const;

	// save playlists to db
	void reset_playlist(int idx);
	PlaylistDBInterface::SaveAsAnswer save_playlist(int idx);
	PlaylistDBInterface::SaveAsAnswer save_cur_playlist();
	PlaylistDBInterface::SaveAsAnswer save_playlist_as(int idx, const QString& name, bool force_override);
	PlaylistDBInterface::SaveAsAnswer save_cur_playlist_as(const QString& name, bool force_override);
	PlaylistDBInterface::SaveAsAnswer rename_playlist(int idx, const QString& name);

	// load playlists from db
	int	load_old_playlists();

	// save playlist as m3u
	void save_playlist_to_file(const QString& filename, bool relative);


private slots:
	// slots triggered by PlayManager
	void played();
	void paused();
	void stopped();
	void previous();
	void next();
	void _sl_playlist_mode_changed();

	void playstate_changed(PlayManager::PlayState);

	// metadata has changed
	void md_changed(const MetaDataList& old_md, const MetaDataList& new_md);
	void md_changed_single(const MetaData&);

private:

	QStringList				_extern_playlists;
	DatabaseConnector*		_db;
	PlaylistDBConnector*	_playlist_db_connector;
	PlayManager*			_play_manager;
	EngineHandler*			_engine;

	QList<PlaylistPtr>		_playlists;

	int						_cur_playlist_idx;
	int						_active_playlist_idx;


	// adds a new playlist, creates it, if name is not in the list of playlists. If name already exists,
	// this function returns the index
	int	add_new_playlist(const QString& name, bool editable, Playlist::Type type=Playlist::Type::Std);

	// raw creation of playlists
	PlaylistPtr new_playlist(Playlist::Type type, int idx, QString name="");

	// checks if a playlist with that name exists, returns idx
	int exists(const QString& name) const;

	// creates new playlist (if neccessary)
	PlaylistPtr get_active();
	PlaylistPtr get_current();

	// called by get_active, get_current
	PlaylistPtr get_playlist(int idx, PlaylistPtr fallback) const;


	// signal wrappers
	void emit_playlist_created(PlaylistPtr pl=nullptr);
	void emit_cur_track_changed(PlaylistPtr pl=nullptr);

	// set active playlist
	void set_active_idx(int idx);

};

#endif /* PLAYLISTHANDLER_H_ */
