/* GUI_TrayIcon.cpp */

/* Copyright (C) 2012  gleugner
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "Helper/Helper.h"
#include "Helper/Style/Style.h"
#include "GUI_TrayIcon.h"

#include <QAction>
#include <QMenu>
#include <QEvent>
#include <QWheelEvent>
#include <QHoverEvent>
#include <QIcon>
#include <QPixmap>
#include <QTimer>
#include <QFont>


GUI_TrayIcon::GUI_TrayIcon (QObject *parent) :

	QSystemTrayIcon (parent),
	NotificationInterface("Standard"),
	SayonaraClass()
{

	_playing = false;
	_muted = false;

	_vol_step = 5;

    _md_set = false;
    
	_play_action = new QAction(tr("Play"), this);
	_play_action->setIcon(Helper::get_icon("play"));
	_stop_action = new QAction(tr("Stop"), this);
	_stop_action->setIcon(Helper::get_icon("stop"));
	_bwd_action = new QAction(tr("Previous"), this);
	_bwd_action->setIcon(Helper::get_icon("bwd"));
	_fwd_action = new QAction(tr("Next"), this);
	_fwd_action->setIcon(Helper::get_icon("fwd"));
	_mute_action = new QAction(tr("Mute"), this);
	_mute_action->setIcon(Helper::get_icon("vol_mute"));
	_show_action = new QAction(tr("Show"), this);
	_close_action = new QAction(tr("Close"), this);
	_close_action->setIcon(Helper::get_icon("power_off"));

	_context_menu = new QMenu();
	_context_menu->addAction(_play_action);
	_context_menu->addAction(_stop_action);
	_context_menu->addSeparator();
	_context_menu->addAction(_fwd_action);
	_context_menu->addAction(_bwd_action);
	_context_menu->addSeparator();
	_context_menu->addAction(_mute_action);
	_context_menu->addSeparator();
	_context_menu->addAction(_show_action);
	_context_menu->addAction(_close_action);

	this->setContextMenu(_context_menu);
	this->setIcon(Helper::get_icon("play"));

	connect(_play_action, &QAction::triggered, this, &GUI_TrayIcon::play_clicked);
	connect(_fwd_action, &QAction::triggered, this, &GUI_TrayIcon::fwd_clicked);
	connect(_bwd_action, &QAction::triggered, this, &GUI_TrayIcon::bwd_clicked);
	connect(_stop_action, &QAction::triggered, this, &GUI_TrayIcon::stop_clicked);
	connect(_close_action, &QAction::triggered, this, &GUI_TrayIcon::close_clicked);
	connect(_mute_action, &QAction::triggered, this, &GUI_TrayIcon::mute_clicked);
	connect(_show_action, &QAction::triggered, this, &GUI_TrayIcon::show_clicked);

	NotificationHandler::getInstance()->register_notificator(this);

	REGISTER_LISTENER(Set::Player_Style, skin_changed);
}


GUI_TrayIcon::~GUI_TrayIcon() {
	delete _play_action;
	delete _stop_action;
	delete _bwd_action;
	delete _fwd_action;
	delete _mute_action;
	delete _close_action;
	delete _show_action;
}


void GUI_TrayIcon::language_changed() {
  
	_play_action->setText(tr("Play"));
	_fwd_action->setText(tr("Next"));
	_bwd_action->setText(tr("Previous"));
	_stop_action->setText(tr("Stop"));
	_mute_action->setText(tr("Mute"));
	_close_action->setText(tr("Close"));
	_show_action->setText(tr("Show"));
}
        

void GUI_TrayIcon::skin_changed() {
	bool dark = (_settings->get(Set::Player_Style) == 1);
	QString stylesheet = Style::get_style(dark);
	_context_menu->setStyleSheet(stylesheet);
    this->setMute(_mute);
}

bool GUI_TrayIcon::event ( QEvent * e ) {

    if (e->type() == QEvent::Wheel) {
        QWheelEvent* wheelEvent = (QWheelEvent*) e;
        emit onVolumeChangedByWheel (wheelEvent->delta());
    }

    return true;
}


void GUI_TrayIcon::notify(const MetaData& md) {

    _md = md;
    _md_set = true;

	if ( isSystemTrayAvailable() ) {

		QString msg = md.title + tr(" by ") + md.artist;
		int timeout = _settings->get(Set::Notification_Timeout);

		showMessage("Sayonara", msg, QSystemTrayIcon::Information, timeout);
	}
}

void GUI_TrayIcon::notify(const QString &title, const QString &message, const QString &image_path) {

	Q_UNUSED(image_path)

	if ( isSystemTrayAvailable() ) {

		int timeout = _settings->get(Set::Notification_Timeout);

		showMessage(title, message, QSystemTrayIcon::Information, timeout);
	}
}

void GUI_TrayIcon::set_enable_play(bool b) {
	_play_action->setEnabled(b);
}

void GUI_TrayIcon::set_enable_stop(bool b) {
	_stop_action->setEnabled(b);
}

void GUI_TrayIcon::set_enable_mute(bool b) {
	Q_UNUSED(b)
}

void GUI_TrayIcon::set_enable_fwd(bool b) {
	_fwd_action->setEnabled(b);
}

void GUI_TrayIcon::set_enable_bwd(bool b) {
	_bwd_action->setEnabled(b);
}

void GUI_TrayIcon::play_clicked() {

	if( !_playing ) {
		emit sig_play_clicked();
	}

	else {
		emit sig_pause_clicked();
	}
}

void GUI_TrayIcon::stop_clicked() {
	emit sig_stop_clicked();
}

void GUI_TrayIcon::stop() {
	_md_set = false;
}

void GUI_TrayIcon::fwd_clicked() {
	emit sig_fwd_clicked();
}

void GUI_TrayIcon::bwd_clicked() {
	emit sig_bwd_clicked();
}

void GUI_TrayIcon::show_clicked(){
	emit sig_show_clicked();
}

void GUI_TrayIcon::close_clicked() {
	emit sig_close_clicked();
}

void GUI_TrayIcon::mute_clicked() {
	emit sig_mute_clicked();
}

void GUI_TrayIcon::setMute(bool mute) {

    _mute = mute;

    QString suffix = "";
	int style = _settings->get(Set::Player_Style);

    if(style == 1) {
        suffix = "_dark";
    }

	if(!mute) {
		_mute_action->setIcon(Helper::get_icon("vol_mute" + suffix));
		_mute_action->setText(tr("Mute"));
	}

	else {
		_mute_action->setIcon(Helper::get_icon("vol_3" + suffix));
		_mute_action->setText(tr("Unmute"));
	}
}

void GUI_TrayIcon::setPlaying(bool play) {

	_playing = play;

	this->setContextMenu(_context_menu);

	if(play) {
		setIcon(Helper::get_icon("play"));

		_play_action->setIcon(Helper::get_icon("pause"));
		_play_action->setText(tr("Pause"));
	}

	else {
		setIcon(Helper::get_icon("pause"));

		_play_action->setIcon(Helper::get_icon("play"));
		_play_action->setText(tr("Play"));
	}
}

int GUI_TrayIcon::get_vol_step() const
{
	return _vol_step;
}
